-- Title: CRC-16 / T10-DIF generation with 8bit parallel input, polynom x^16 + x^15 + x^11 + x^9 + x^8 + x^7 + x^5 + x^4 + x^2 + x^1 + 1, input not reversed, output not reversed
-- Copyright notice:  (C) 2017 www.dlsconsulting.de
-- Usage and distribution of this source file is free, when the copyright notice is not removed
-- Note, that no warranty on completeness or correctness can be taken over for this source code

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package crc_definitions is
   function NewCRC(OldCRC: std_logic_vector(15 downto 0); DtIn: std_logic_vector(7 downto 0)) return std_logic_vector;
   constant CRC16_Ini     : std_logic_vector(15 downto 0) := x"0000";    -- CRC start value
   constant CRC16_Fin     : std_logic_vector(15 downto 0) := x"0000";    -- CRC final XOR value
end package crc_definitions;

package body crc_definitions is
   function NewCRC(OldCRC: std_logic_vector(15 downto 0); DtIn: std_logic_vector(7 downto 0)) return std_logic_vector is
      variable crc : std_logic_vector(15 downto 0);
   begin
      crc(0) := OldCRC(15) xor OldCRC(14) xor OldCRC(12) xor OldCRC(11) xor OldCRC(10) xor OldCRC(9) xor OldCRC(8) xor DtIn(7) xor DtIn(6) xor DtIn(4) xor DtIn(3) xor DtIn(2) xor DtIn(1) xor DtIn(0);
      crc(1) := OldCRC(14) xor OldCRC(13) xor OldCRC(8) xor DtIn(6) xor DtIn(5) xor DtIn(0);
      crc(2) := OldCRC(12) xor OldCRC(11) xor OldCRC(10) xor OldCRC(8) xor DtIn(4) xor DtIn(3) xor DtIn(2) xor DtIn(0);
      crc(3) := OldCRC(13) xor OldCRC(12) xor OldCRC(11) xor OldCRC(9) xor DtIn(5) xor DtIn(4) xor DtIn(3) xor DtIn(1);
      crc(4) := OldCRC(15) xor OldCRC(13) xor OldCRC(11) xor OldCRC(9) xor OldCRC(8) xor DtIn(7) xor DtIn(5) xor DtIn(3) xor DtIn(1) xor DtIn(0);
      crc(5) := OldCRC(15) xor OldCRC(11) xor OldCRC(8) xor DtIn(7) xor DtIn(3) xor DtIn(0);
      crc(6) := OldCRC(12) xor OldCRC(9) xor DtIn(4) xor DtIn(1);
      crc(7) := OldCRC(15) xor OldCRC(14) xor OldCRC(13) xor OldCRC(12) xor OldCRC(11) xor OldCRC(9) xor OldCRC(8) xor DtIn(7) xor DtIn(6) xor DtIn(5) xor DtIn(4) xor DtIn(3) xor DtIn(1) xor DtIn(0);
      crc(8) := OldCRC(13) xor OldCRC(11) xor OldCRC(8) xor OldCRC(0) xor DtIn(5) xor DtIn(3) xor DtIn(0);
      crc(9) := OldCRC(15) xor OldCRC(11) xor OldCRC(10) xor OldCRC(8) xor OldCRC(1) xor DtIn(7) xor DtIn(3) xor DtIn(2) xor DtIn(0);
      crc(10) := OldCRC(12) xor OldCRC(11) xor OldCRC(9) xor OldCRC(2) xor DtIn(4) xor DtIn(3) xor DtIn(1);
      crc(11) := OldCRC(15) xor OldCRC(14) xor OldCRC(13) xor OldCRC(11) xor OldCRC(9) xor OldCRC(8) xor OldCRC(3) xor DtIn(7) xor DtIn(6) xor DtIn(5) xor DtIn(3) xor DtIn(1) xor DtIn(0);
      crc(12) := OldCRC(15) xor OldCRC(14) xor OldCRC(12) xor OldCRC(10) xor OldCRC(9) xor OldCRC(4) xor DtIn(7) xor DtIn(6) xor DtIn(4) xor DtIn(2) xor DtIn(1);
      crc(13) := OldCRC(15) xor OldCRC(13) xor OldCRC(11) xor OldCRC(10) xor OldCRC(5) xor DtIn(7) xor DtIn(5) xor DtIn(3) xor DtIn(2);
      crc(14) := OldCRC(14) xor OldCRC(12) xor OldCRC(11) xor OldCRC(6) xor DtIn(6) xor DtIn(4) xor DtIn(3);
      crc(15) := OldCRC(14) xor OldCRC(13) xor OldCRC(11) xor OldCRC(10) xor OldCRC(9) xor OldCRC(8) xor OldCRC(7) xor DtIn(6) xor DtIn(5) xor DtIn(3) xor DtIn(2) xor DtIn(1) xor DtIn(0);
      return crc;
   end NewCRC;
end package body;